import java.io.*;
import java.util.*;

/**
*Trida Text
*analyzuje a pracuje s textem
*@author M.Kales
*/
public class Text{
	
	public String soubor;
	public Strom strom;
	public BufferedReader bfr;
	public String chyba;
	public PrintWriter pw;
	public char pismeno;
	private static String platneZnakySlova = "[^A-Za-z켾]+";
	public int maxCetnost;
	public PrvekCetnost[] cetnost;
	public PrvekCetnost nynejsi;
	public Prvek ukazatel;
	public String[] Vysledky, pomoc;

	/**
	 *konstruktor tridy Text
	 *@param bfr	otevreny vstupni soubor
	 */	
	public Text(BufferedReader bfr){		
		this.strom=zpracujText(bfr);
		maxCetnost=0;
		cetnost = new PrvekCetnost[0];
	}
	
	/**
	 *metoda na zpracovani textu,
	 *projde cely vstupni soubor, rozdeli na slova a prida do BVS
	 *@param text	otevreny vstupni soubor
	 *@return zpracovany text v BVS
	 */		
	public Strom zpracujText(BufferedReader text){
		try{
			String radka, slovo;
			int a=0;
			int b=0;
			Strom textStrom=new Strom();
			String[] slova;
			while((radka=text.readLine())!=null){
				radka=radka.toLowerCase();
				slova = radka.split(platneZnakySlova);
				for (int i = 0; i < slova.length; i++) {
					slova[i] = slova[i].trim().toLowerCase();
					if (slova[i].length() != 0)
					{
						textStrom.pridejPrvek(new Prvek(slova[i]));
					}
				}		
			}	
			return textStrom;		
		}
		catch (Exception e) {
			System.out.println("Chyba ve vstupnim souboru!!!");
			e.printStackTrace(); 
		}
		return null;
	}
	
	/**
	 *metoda pro vypis prvku abecedne s cetnosti,
	 *vypisuje do HTML
	 */		
	public void SeraditAbecedne(){
		try{
			pw = new PrintWriter(new BufferedWriter(new FileWriter("out_"+soubor+".html")));
			pw.println("<html><head><title>" + soubor + "</title><style>table {border: 2px solid black} th {border: 2px solid gray; padding: 4px} td {border: 1px solid gray; padding: 4px; text-align: center}</style></head><body><table><tr><th>etnost</th><th>Slovo</th></tr>");
			inOrder(strom.koren);
			pw.println("</table></body></html>");
			pw.close();
		}
		catch(IOException e){
		}
		return;
	}
	
	/**
	 *metoda pro prohledavani BVS - inOrder
	 *vypisuje do HTML (podle abecedy)
	 *@param pointer	prvek od ktereho zacne prohledavani
	 */	
	public void inOrder(Prvek pointer) {
		if (pointer == null){
			return;		
		}
		inOrder(pointer.vlevo);
		pismeno=pointer.hodnota.charAt(0);
		pw.println("<tr><td>" + pointer.cetnost + "</td><td>" + pointer.hodnota + "</td></tr>");
		inOrder(pointer.vpravo);
	}
	
	/**
	 *metoda pro vypis prvku podle cetnosti,
	 * prevadi prvky z BVS do Hash tabulky
	 *vypisuje do HTML
	 */	
	public void SeraditCetnost(){
		inOrderCetMax(strom.koren);
		cetnost = new PrvekCetnost[maxCetnost];
		inOrderCetnost(strom.koren);
		try{
			pw = new PrintWriter(new BufferedWriter(new FileWriter("out_Cetnost_"+soubor+".html")));
			pw.println("<html><head><title>" + soubor + "</title><style>table {border: 2px solid black} th {border: 2px solid gray; padding: 4px} td {border: 1px solid gray; padding: 4px; text-align: center}</style></head><body><table><tr><th>etnost</th><th>Slovo</th></tr>");
			for (int i=maxCetnost-1; i>=0; i--){
				if (cetnost[i]!=null){
					nynejsi=cetnost[i];
					while (nynejsi!=null){
						pw.println("<tr><td>" + nynejsi.prvek.cetnost + "</td><td>" + nynejsi.prvek.hodnota + "</td></tr>");
						nynejsi=nynejsi.dalsi;
					}
				}
			}
			pw.println("</table></body></html>");
			pw.close();
		}
		catch(IOException e){
		}
		return;
	}
	
	/**
	 *metoda pro prohledavani BVS - inOrder
	 *zjistuje nejvetsi cetnost
	 *@param pointer	prvek od ktereho zacne prohledavani
	 */	
	public void inOrderCetMax(Prvek pointer) {
		if (pointer == null){
			return;		
		}
		inOrderCetMax(pointer.vlevo);
		if (maxCetnost<pointer.cetnost){
			maxCetnost=pointer.cetnost;
		}
		inOrderCetMax(pointer.vpravo);
	}
	
	/**
	 *metoda pro prohledavani BVS - inOrder
	 *vytvari Hash tabulku a uklada prvky podle cetnosti
	 *@param pointer	prvek od ktereho zacne prohledavani
	 */		
	public void inOrderCetnost(Prvek pointer) {
		if (pointer == null){
			return;		
		}
		inOrderCetnost(pointer.vlevo);
		nynejsi=cetnost[pointer.cetnost-1];
		if (cetnost[pointer.cetnost-1]==null){
			cetnost[pointer.cetnost-1]=new PrvekCetnost(pointer);
		}
		else{
			while(nynejsi.dalsi!=null){
				nynejsi=nynejsi.dalsi;
			}
			nynejsi.dalsi=new PrvekCetnost(pointer);
		}
		inOrderCetnost(pointer.vpravo);
	}
	
	/**
	 *metoda pro vyhledavani
	 *@param klic	hledane slovo
	 *@return pole nalezenych slov
	 */		
	public String[] Hledej (String klic){
		klic=klic.toLowerCase();
		inOrderVypis(klic, strom.koren);
		return Vysledky;
	}
	
	/**
	 *metoda pro prohledavani BVS - inOrder
	 *prohledava BVS a hleda slova ktera zacinaji zadanym klicem
	 *@param pointer	prvek od ktereho zacne prohledavani
	 *@param klic	vyhledavane slovo
	 */		
	public void inOrderVypis(String klic, Prvek pointer){
		if (pointer == null){
			return;		
		}
		inOrderVypis(klic, pointer.vlevo);
		if (klic.length()<=pointer.hodnota.length()){
			if (klic.equals(pointer.hodnota.substring(0,klic.length()))){		
				if (Vysledky==null){
					Vysledky=new String [1];
					Vysledky[0]=pointer.hodnota;
				}
				else{
					pridejVysledek(pointer.hodnota);
				}
			}
		}
		inOrderVypis(klic, pointer.vpravo);
	}
	
	/**
	 *metoda pro pridani nalezeneho slova do globalni promene
	 *@param dalsi	pridavane slovo
	 */		
	private void pridejVysledek(String dalsi){
		pomoc=Vysledky;
		Vysledky=new String[pomoc.length+1];
		for (int i=0; i<pomoc.length; i++){
			Vysledky[i]=pomoc[i];
		}
		Vysledky[pomoc.length]=dalsi;
	}

	/**
	 *privatni trida PrvekCetnosti,
	 *prvek Hash tabulky
	 *@author M.Kales
	 */	
	private class PrvekCetnost{
		
		PrvekCetnost dalsi;
		Prvek prvek;
		
		/**
		*konstruktor privatni tridy PrvekCetnosti
		*@param prvek	prvek z BVS
		*/		
		PrvekCetnost(Prvek prvek){
			this.prvek=prvek;
		}
	}
}